#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#


import unittest
import subprocess
import StringIO


class testcase(object):
    def __init__(self, argv, expect_exitcode, expect_stdout, expect_stderr=''):
        self._argv = argv
        self._expect_exitcode = expect_exitcode
        self._expect_stdout = expect_stdout
        self._expect_stderr = expect_stderr

    def test_commandline(self):
        stdout = StringIO.StringIO()
        stderr = StringIO.StringIO()
        exitcode = subprocess.call(argv, stdout=stdout, stderr=stderr)
        self.assertEqual(exitcode, self._expect_exitcode)
        self.assertEqual(stdout, self._expect_stdout)
        self.assertEqual(stderr, self._expect_stderr)

class Test_iprange(unittest.TestCase, CommandlineTester):
    def __init__(self):
        unittest.TestCase.__init__(self)


def _test():
    import doctest
    import unittest
    doctest.testmod()
    unittest.main()

if __name__ == "__main__":
    if 0:  # DISABLED
        _test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#


import sys
import os
import os.path
import optparse
import itertools
import time
import fcntl

import gevent
import gevent.pool
import gevent.monkey
from gevent.socket import wait_read

import xnet.tools


gevent.monkey.patch_all(thread=False)

DEVELOPMENT = 1

if DEVELOPMENT:
    #
    # FIXME: FULHACK dev
    #
    import os
    from os.path import join, abspath
    sys.path.insert(0, abspath(join(os.getcwd(), '../../')))

import xnet.debug


class XNetOptionParser(optparse.OptionParser):
    '''
        Provides some common options and an interface to add nicely formatted
        local options for each tool. Each local option is defined as a
        four-tuple of (shortoption, longoption, helptext, keyword-dict).

        Example follows below:

        class LocalOptionParser(OptionParserBase):
            local_options = [
                ('-p', '--port', 'Port to state.',
                    dict(dest='port'))
            ]
    '''
    def __init__(self, local_options, **kw):
        optparse.OptionParser.__init__(self, **kw)
        self.add_option('', '--test', dest='test', action='store_true',
                        help='run tests')
        self.add_option('', '--pdb', dest='pdb', action='store_true',
                        help='enable interactive pdb debugging on exceptions')
        self.add_option('', '--print-source', dest='print_source', action='store_true',
                        help='print source code of tool')
        self.add_option('-y', '--parallelism', dest='parallelism', metavar='n',
                        help='set parallelism (default 128)')
        self.add_option('-r', '--read', dest='read', metavar='path',
                        help='read input from file')
        self.add_option('-w', '--wait', dest='wait', metavar='nsecs',
                        help='wait at most nsecs seconds for actions to complete')
        self.add_option('-i', '--interval', dest='interval', metavar='nsecs',
                        help='time interval between each action ')
        self.add_option('', '--repeat', dest='repeat', metavar='ntimes',
                        help='repeat action this many times (-1 = infinity)')
        self.add_option('', '--time', dest='time', action='store_true',
                        help='print time consumed by action')
        self.add_option('-T', '--supress-timeout', dest='supress_timeout',
                        help='supress messages from tasks timing out', action='store_true')
        self.add_option('-f', '--format', dest='format', metavar='fmt',
                        help='specify output format, see --format-help for info')
        self.add_option('', '--format-help', dest='format_help', action='store_true',
                        help='print output format help')
        self.add_option('', '--split-output', dest='split_output', metavar='base',
                        help='produce one output file for each piece of input')
        self.add_option('', '--split-tee', dest='split_tee', metavar='base',
                        help='like --split-output but also print to stdout')
        self.add_option('-v', '--verbose', dest='verbose',
                        help='verbose output', action='store_true')
        self._load_local_options(local_options)

    def _load_local_options(self, local_options):
        for (short_, long_, help_, kw) in local_options:
            help_ = '* ' + help_  # mark tool-unique options
            self.add_option(short_, long_, help=help_, **kw)


def get_action_usage():
    _action_usage = ''
    for tup in _available_actions:
        name = tup[0]
        desc = available_actions[name].__description__
        line = ' '
        line += name
        line += ' ' * (11 - len(name))
        line += desc
        line += '\n'
        _action_usage += line
    return _action_usage


__usage__ = """

    $$$ Merry Xnet! $$$

 $ {{tool}} [options] item1 item2 ...
 $ {{tool}} [options] -r itemfile.txt
 $ cat itemsfile.txt | {{tool}} [options] -
"""


def top_usage():
    output = __usage__
    output += '\n'
    output += 'Available tools:\n'
    output += get_action_usage()
    return output


def print_error(a):
    errmsg = str(a.error)
    if not errmsg.strip():
        errmsg = 'no-error-set\n'
    if not errmsg[-1] == '\n':
        errmsg += '\n'
    item = str(a._item).strip()
    errmsg = item + ' ' + errmsg
    sys.stderr.write(errmsg)


def stdin_disperser_greenlet(pool):
    '''
        loop forever feeding active greenlets with data
        read from stdin.
    '''
    done = False
    fcntl.fcntl(sys.stdin, fcntl.F_SETFL, os.O_NONBLOCK)
    while not done:
        wait_read(sys.stdin.fileno())
        try:
            data = sys.stdin.read()
        except:
            raise
        done = (data == '')
        for greenlet in pool:
            obj = greenlet._run  # fulhack
            if not hasattr(obj, '__massage__'):  # fulhack
                continue
            #obj.feed_stdin(data)  # put in queue or list
            os.write(obj._stdin_pipe[1], data)


def _get_output_filepath(a, base):
    #
    # item is the result of some specific processing
    # and may contain arbitrary meta characters
    # which should not affect the file system path
    # where the output file is created.
    #
    path = base + a.__filename__()
    return path


def repeaterator(it, count):
    '''
        repeat iterator count times or -1 for infinity
    '''
    assert type(count) is int
    cpy = []
    if count != 0:
        for item in it:
            cpy.append(item)
            yield item
        if count > 0:
            count -= 1
    while count != 0:
        for item in cpy:
            yield item
        if count > 0:
            count -= 1


def run(ToolImplementation, tool_reads_stdin=False):
    try:
        return _run(ToolImplementation, tool_reads_stdin)
    except KeyboardInterrupt:
        return 1


def _run(ToolImplementation, tool_reads_stdin):
    '''
            tool_reads_stdin - tool uses stdin for other purpose than
                               to obtain startup arguments
    '''
    args = sys.argv[:]
    action_name = args[0]
    #args = list(args)[1:]

    _E_INVALID_ARGUMENT_STDIN = \
        '-: Invalid argument, target hosts may not be specified on stdin.'

    _usage = __usage__.replace('{{tool}}', action_name)
    if hasattr(ToolImplementation, '__itemname__'):
        itemname = ToolImplementation.__itemname__
        _usage = _usage.replace('item', itemname)
    parser = XNetOptionParser(ToolImplementation.cmdline_options, usage=_usage)
    (opt, args) = parser.parse_args(args)
    cmdlineinputs = args[1:]

    if opt.pdb:
        xnet.debug.interactive_debugger_on_exception(True)

    if opt.print_source:
        ToolImplementation.print_source()
        return

    #
    # chain input sources, handle options
    # precedence order: cmdline, -r, stdin
    #
    _inputs = []
    if '-' in cmdlineinputs:
        if tool_reads_stdin:
            raise Exception(_E_INVALID_ARGUMENT_STDIN)
        cmdlineinputs.remove('-')
        _inputs.append(sys.stdin)
    if opt.read:
        f = open(opt.read)
        _inputs.insert(0, f)
    if len(cmdlineinputs):
        _inputs.insert(0, cmdlineinputs)
    inputchain = itertools.chain(*_inputs)

    _parallelism = 128
    if not opt.parallelism is None:
        _parallelism = int(opt.parallelism)

    _wait = None
    if not opt.wait is None:
        _wait = float(opt.wait)

    _interval = 0.0
    if not opt.interval is None:
        _interval = float(opt.interval)

    _repeat = 1
    if not opt.repeat is None:
        _repeat = int(opt.repeat)

    _outfile = sys.stdout
    #
    # main loop
    #

    if opt.format_help:
        print ToolImplementation.__format_help__()
        sys.exit(0)

    if opt.split_tee:
        opt.split_output = opt.split_tee

    #
    # verify commandline options and do preparations
    #
    ToolImplementation.__setup__(opt)

    pool = gevent.pool.Pool(_parallelism)
    actions = []  # all spawned actions
    greenlets = []

    wkpool = None
    if _wait:
        wkpool = gevent.pool.Pool(_parallelism)

    def waitkill(g, killtime):
        sleeptime = killtime - time.time()
        #print 'waitkill', sleeptime
        if sleeptime > 0:
            gevent.sleep(sleeptime)
        if not g.ready():
            gevent.kill(g, xnet.tools.WaitTimeout)
            g.join()

    inputchain = ToolImplementation.__massage__(inputchain, opt)

    kwargs = {}

    if tool_reads_stdin:
        gevent.spawn(stdin_disperser_greenlet, pool)


    _t = 0.0
    killtime = 0

    if _repeat != 1:
        inputchain = repeaterator(inputchain, _repeat)

    for (i, line) in enumerate(inputchain):
        action = ToolImplementation(opt, **kwargs)
        pool.wait_available()
        if _wait:
            killtime = time.time() + _wait
        g = pool.spawn(action, line, inputchain)
        g.action = action
        greenlets.append(g)
        #
        # Timeout seems unreliable on debian squeeze, use waitkill instead.
        #
        if _wait:
            wkpool.wait_available()
            wk = wkpool.spawn(waitkill, g, killtime)

        #
        # handle finished actions
        #
        while len(greenlets) and greenlets[0].ready():
            action = greenlets[0].action
            output_action_result(action, opt, _outfile)
            del action
            del greenlets[0]

        #
        # handle interval
        #
        _this_interval = _interval - (time.time() - _t)
        if _this_interval > 0:
            gevent.sleep(_this_interval)
        _t = time.time()

    if wkpool:
        wkpool.join()
    gevent.joinall(greenlets) #, timeout=timeout)
    not_done = filter(lambda g: (not g.ready()), greenlets)
    if len(not_done) > 0 and not _wait is None:
        print 'ERROR: not_done has contents inspite of _wait and grace time'
        print not_done

    #
    # Force-kill greenlets that didn't die in spite of 1 sec of grace time.
    #
    gevent.killall(not_done, block=True)
    gevent.joinall(not_done)

    #
    # print results
    #
    for g in greenlets:
        action = g.action
        output_action_result(action, opt, _outfile)
        del action

def output_action_result(a, opt, _outfile):
    outfile = _outfile
    if opt.split_output:
        path = _get_output_filepath(a, opt.split_output)
        outfile = open(path, 'wb')
    if not a.error is None:
        if a.error == xnet.tools.WaitTimeout:
            if not opt.supress_timeout:
                #
                # __timeout__() messages always go to stderr.
                #
                msg = a.__timeout__()
                if not msg is None:
                    sys.stderr.write(msg)
        else:
            print_error(a)
    else:
        if a._action_result:
            msg = str(a)
            #
            # ignore empty output from Tool.__format__()
            #
            if len(msg) == 0:
                return
            msg += '\n'
            if a.stderr:
                outfile = sys.stderr
            outfile.write(msg)
            if opt.split_tee and not a.stderr:
                _outfile.write(msg)

    _outfile.flush()
    if not outfile is _outfile:
        outfile.flush()

    if opt.split_output:
        outfile.close()


__xnet_usage__ = '''

    $$$ Merry Xnet! $$$

The following xnet tools are available in your PATH, if installation succeeded:
 xnet      - xnet meta tool
 iprange   - IP range enumerator
 resolv    - hostname resolver
 tcpstate  - port scanner
 netcats   - multi-peer netcat client
 webget    - web client
 sslinfo   - SSL info harvester
 iissn     - IIS shortname enumeration

Help for some specific tool:
 $ tcpstate --help

Analogous commands:
 $ iprange ...
 $ xnet iprange ...
 $ iprange --print-source > foo.py && xnet foo.py

Develop your own, or make alternative versions of existing xnet tools:
 $ existing_xnet_tool --print-source > newtool.py
 $ vim newtool.py
 $ xnet newtool.py ...
'''


def main():
    args = sys.argv[:]

    if len(args) < 2 or args[1] in ('-h', '--help'):
        print __xnet_usage__
        return

    ToolImplementation = None

    toolname = args[1]
    if toolname.endswith('.py'):
        if not toolname.startswith('/'):
            toolname = os.path.abspath(toolname)
        if not os.path.exists(toolname):
            print 'File not found:', toolname
        folder = os.path.dirname(toolname)
        if len(folder):
            sys.path.insert(0, folder)
        modulename = os.path.basename(toolname).rsplit('.', 1)[0]
        toolmodule = __import__(modulename)
        sys.path = sys.path[1:]
        for name in dir(toolmodule):
            val = getattr(toolmodule, name)
            found = False
            try:
                if issubclass(val, xnet.tools.Tool):
                    found = True
            except TypeError:
                pass
            if found and not val is xnet.tools.Tool:
                if not ToolImplementation is None:
                    errmsg = 'Multiple Tool implementations in {0}: {1}, {2}'
                    errmsg = errmsg.format(toolname, ToolImplementation, val)
                    print errmsg
                    sys.exit(1)
                ToolImplementation = val

    if not ToolImplementation:
        if toolname == 'iissn':
            from xnet.tools.iissn import IISShortnameTool as ToolImplementation
        if toolname == 'iprange':
            from xnet.tools.iprange import IPRangeTool as ToolImplementation
        if toolname == 'netcats':
            from xnet.tools.netcats import NetcatsTool as ToolImplementation
        if toolname == 'resolv':
            from xnet.tools.resolv import ResolvTool as ToolImplementation
        if toolname == 'sslinfo':
            from xnet.tools.sslinfo import SSLInfoTool as ToolImplementation
        if toolname == 'tcpstate':
            from xnet.tools.tcpstate import TcpStateTool as ToolImplementation
        if toolname == 'webget':
            from xnet.tools.webget import WebgetTool as ToolImplementation

    if not ToolImplementation:
        print 'Tool not found:', toolname
        sys.exit(1)

    sys.argv = sys.argv[1:]

    result = run(ToolImplementation)
    return result


if __name__ == "__main__":
    main()
    #if '--test' in sys.argv:
    #    sys.argv.remove('--test')
    #    doctest.testmod()
    #    unittest.main()
    #else:
    #    main(*sys.argv)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2011 Krister Hedfors
#

__all__ = ['SSLInfoTool']


import unittest
import doctest
import sys
import ssl
import socket

import OpenSSL
import gevent

from xnet.tools import Tool
from xnet.net.ipv4 import IPRangeIterator

from xnet.debug import pdb


class SSLInfoException(Exception):
    pass


class SSLInfoTool(Tool):

    __toolname__ = 'sslinfo'
    __itemname__ = 'host'
    __description__ = 'get ssl and certificate information'

    cmdline_options = [
        ('-p', '--port', 'tcp port',
            dict(dest='port')),
        ('', '--cn', 'print common name only',
            dict(dest='cn', action='store_true'))
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    @classmethod
    def __massage__(self, iterator, options):
        for item in iterator:
            item = item.strip()
            try:
                iprangeiter = IPRangeIterator(item)
            except ValueError:
                yield item
            else:
                for ip in iprangeiter:
                    yield str(ip)

    def __parse__(self, line, iterator):
        result = {}
        host = line.strip()
        port = 443
        if '://' in line:
            proto, host = line.split('://', 1)
            if proto != 'https':
                raise SSLInfoException('unknown uri protocol: ' + line)
            if host[-1] == '/':
                host = host[:-1]
        if self.options.port:
            port = int(self.options.port)
        addr = (host, port)
        result['addr'] = addr
        return result

    def __action__(self, parse_result):
        result = {}
        self._addr = addr = parse_result['addr']
        try:
            cert = ssl.get_server_certificate(addr)
            cert = self._fix_cert(cert)
            x509 = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, cert)
        except socket.error, e:
            #
            # errno numbers differ between platforms,
            # but tests for 'closed' and 'filtered'
            # below should work universally.
            #
            errmsg = e.strerror.lower()
            if 'connection refused' in errmsg:
                errmsg = 'closed {0} {1}'.format(*addr)
            elif 'timed out' in errmsg:
                errmsg = 'filtered {0} {1}'.format(*addr)
            else:
                errmsg = errmsg.replace(' ', '-').replace(',', '-')
            self.set_error(errmsg)
            self.stderr = True
            return None
        except Exception, e:
            errmsg = addr[0] + ' ' + str(e) + '\n'
            self.set_error(errmsg)
            self.stderr = True
            return None
        result['host'] = addr[0]
        result['port'] = addr[1]
        result['cert'] = cert
        result['x509'] = x509
        result['notbefore'] = x509.get_notBefore()
        result['notafter'] = x509.get_notAfter()
        result['serial'] = str(x509.get_serial_number()).replace('L', '')
        issuer = x509.get_issuer().get_components()
        result['issuer'] = ''.join('/' + n + '=' + v for (n, v) in issuer)
        subject = x509.get_subject().get_components()
        result['subject'] = ''.join('/' + n + '=' + v for (n, v) in subject)
        subject = dict(subject)
        result['C'] = subject.get('C', '-')
        result['ST'] = subject.get('ST', '-')
        result['O'] = subject.get('O', '-')
        result['OU'] = subject.get('OU', '-')
        result['CN'] = subject.get('CN', '-')
        result['email'] = subject.get('emailAddress', '-')
        return result

    def _fix_cert(self, cert):
        '''
            fix bug which appears on BT5R3 but not OSX Lion
        '''
        assert '-----END CERTIFICATE' in cert
        if not '\n-----END CERTIFICATE' in cert:
            cert = cert.replace('-----END CERTIFICATE',
                                '\n-----END CERTIFICATE')
        return cert

    def __format__(self, line, parse_result, action_result):
        output = ''
        host, port = parse_result['addr']
        x509 = action_result['x509']
        cd = dict(x509.get_subject().get_components())
        #cd = action_result['components_dict']
        if self.options.cn:
            output = '{0} {1}'.format(host, cd.get('CN', None))
        else:
            for (name, val) in cd.iteritems():
                output += '{0} {1}={2}\n'.format(host, name, val)
            if output[-1] == '\n':
                output = output[:-1]
        return output

    def __timeout__(self):
        return 'timeout {0} {1}\n'.format(*self._addr)

    @classmethod
    def __format_help__(cls):
        name = cls.__toolname__
        output = '''
Format variables available for %s.
(default format is complex and not reproducable from commandline)

 host        - target host
 port        - target port
 notbefore   - not valid before
 notafter    - not valid after
 serial      - cert serial number
 issuer      - issuer common name, organizational unit, ...
 subject     - subject common name, organizational unit, ...
 C           - subject country
 ST          - subject state
 O           - subject organization
 OU          - subject organizational unit
 CN          - subject common name
 email       - subject email address
 time        - time consumed by action, in ms
        ''' % cls.__toolname__
        return output


def main():
    import xnet.tools.run
    xnet.tools.run.run(SSLInfoTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2011 Krister Hedfors
#

__all__ = ['IISShortnameTool']


import unittest
import doctest

from webget import WebgetTool
from xnet.net.ipv4 import IPRangeIterator


class IISShortnameTool(WebgetTool):

    __description__ = 'identify IIS shortname functionality'
    __toolname__ = 'iissn'
    __itemname__ = 'url'

    cmdline_options = [
        ('-p', '--ports', 'use these ports for urls lacking port AND protocol components',
            dict(dest='ports', metavar='m-n,..')),
        ('-s', '--server', 'print server header',
            dict(dest='server', action='store_true')),
        ('-S', '--https', 'default to https when protocol is missing',
            dict(dest='https', action='store_true')),
        ('', '--host', 'specify Host header manually',
            dict(dest='host')),
        ('', '--method', 'specify alternative method. Examples GET, HEAD, POST, ...',
            dict(dest='method', metavar='method')),
        ('', '--proxy', 'not working, sorry!',
            dict(dest='proxy')),
        ('', '--redirect-allow-samedomain', 'follow redirects under the same domain',
            dict(dest='redirect_allow_samedomain', action='store_true')),
        ('', '--redirect-whitelist', 'comma-separated whitelist of allowed redirect hosts',
            dict(dest='redirect_whitelist', metavar='hosts')),
        ('', '--code', 'only print response code',
            dict(dest='code', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    def __init__(self, *args, **kw):
        super(IISShortnameTool, self).__init__(*args, **kw)
        self._url = None

    #@classmethod
    #def __massage__(self, iterator, options):
    #    for item in iterator:
    #        item = item.strip()
    #        try:
    #            iprangeiter = IPRangeIterator(item)
    #        except ValueError:
    #            yield item
    #        else:
    #            for ip in iprangeiter:
    #                yield str(ip)

    def __parse__(self, line, iterator):
        result = super(IISShortnameTool, self).__parse__(line, iterator)
        self._url = url = result['url']
        if not url.endswith('/'):
            url += '/'
        url1 = url + '*~1*/.aspx'
        url2 = url + 'poipoipoi*~1*/.aspx'
        result['url'] = url
        result['url1'] = url1
        result['url2'] = url2
        return result

    def __action__(self, parse_result):
        '''
            v0 = url
            v1 = vulnerable
        '''
        host = parse_result['host']
        url = parse_result['url']
        url1 = parse_result['url1']
        url2 = parse_result['url2']
        result1 = super(IISShortnameTool, self).__action__(
            {'host': host, 'url': url1})
        result2 = super(IISShortnameTool, self).__action__(
            {'host': host, 'url': url2})
        vulnerable = False
        code1 = result1['code']
        code2 = result2['code']
        if type(code1) is int and type(code2) is int:
            vulnerable = code1 != code2
        result = {}
        result['host'] = host
        result['url'] = result['v0'] = url
        result['vulnerable'] = result['v1'] = vulnerable
        result['url1'] = url1
        result['url2'] = url2
        result['code1'] = result1['code']
        result['code2'] = result2['code']
        result['msg1'] = result1['msg']
        result['msg2'] = result2['msg']
        result['body1'] = result1['body']
        result['body2'] = result2['body']
        return result

    def __format__(self, line, parse_result, value):
        host = value['host']
        url = value['url']
        vulnerable = value['vulnerable']
        code1 = value['code1']
        code2 = value['code2']
        output = ''
        #
        #if options.server:
        #elif options.code:
        if self.options.verbose or self.options.code:
            output = '{0} {1} {2} {3} {4}'.format(
                url,
                host,
                vulnerable,
                code1,
                code2,
            )
        else:
            output = '{0} {1}'.format(
                url,
                vulnerable,
            )
        return output

    @classmethod
    def __format_help__(cls):
        name = cls.__toolname__
        output = '''
Format variables for %s:

 url         - url to probe
 host        - Host header
 vulnerable  - boolean result
 code1       - response code from url+'*~1*/.aspx'
 code2       - response code from url+'poipoipoi*~1*/.aspx'
 time        - time consumed by action, in ms

Default format:
 '{url} {host} {vulnerable} {code1} {code2}'

Default format if verbose (-v):
 '{url} {vulnerable}'
        ''' % cls.__toolname__
        return output

    def __timeout__(self):
        return 'timeout {0}\n'.format(self._url)


def main():
    import xnet.tools.run
    xnet.tools.run.run(IISShortnameTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2011 Krister Hedfors
#


import unittest
import doctest
import socket

import gevent

from xnet.tools import Tool
from xnet.net.ipv4 import IPRangeIterator


class ResolvTool(Tool):

    __toolname__ = 'resolv'
    __itemname__ = 'host'
    __description__ = 'resolve hostnames to IPs back and forth'

    cmdline_options = [
        ('-e', '--reverse', 'perform reverse lookup',
            dict(dest='reverse', action='store_true')),
        ('-c', '--deep', 'keep looking up as long as new addresses appear',
            dict(dest='deep', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    @classmethod
    def __massage__(self, iterator, options):
        for item in iterator:
            item = item.strip()
            iprit = None
            try:
                iprit = IPRangeIterator(item)
            except ValueError:
                yield item
            else:
                for ip in iprit:
                    yield ip

    def __parse__(self, line, iterator):
        self._item = item = line.strip()
        result = {}
        if self.options.reverse:
            result['ip'] = item
        else:
            result['host'] = item
        return result

    def __action__(self, parse_result):
        if self.options.reverse:
            return self._reverse_lookup(parse_result)
        else:
            return self._lookup(parse_result)

    def _lookup(self, parse_result):
        host = parse_result['host']
        result = {'host': host}
        try:
            ip = socket.gethostbyname(host)
            result['ip'] = ip
        except socket.error, e:
            #
            # errno numbers differ between platforms,
            # but tests for 'closed' and 'filtered'
            # below should work universally.
            #
            errmsg = e.strerror.lower()
            if 'connection refused' in errmsg:
                errmsg = 'closed'
            elif 'timed out' in errmsg:
                errmsg = 'filtered'
            else:
                errmsg = errmsg.replace(' ', '-').replace(',', '-')
            errmsg = '{0} {1}'.format(host, errmsg)
            self.set_error(errmsg)
            self.stderr = True
            return None
        #except gevent.GreenletExit:
        #    result['state'] = 'timeout'
        finally:
            pass
        return result

    def _reverse_lookup(self, parse_result):
        ip = parse_result['ip']
        result = {'ip': ip}
        try:
            addrlist = socket.gethostbyaddr(ip)
            result['host'] = addrlist[0]
            result['hostlist'] = addrlist
        except socket.error, e:
            #
            # errno numbers differ between platforms,
            # but tests for 'closed' and 'filtered'
            # below should work universally.
            #
            errmsg = e.strerror.lower()
            if 'connection refused' in errmsg:
                errmsg = 'closed'
            elif 'timed out' in errmsg:
                errmsg = 'filtered'
            else:
                errmsg = errmsg.replace(' ', '-').replace(',', '-')
            errmsg = '{0} {1}'.format(ip, errmsg)
            self.set_error(errmsg)
            self.stderr = True
            return None
        #except gevent.GreenletExit:
        #    result['state'] = 'timeout'
        finally:
            pass
        return result

    def __format__(self, line, parse_result, action_result):
        output = '{0} {1}'.format(
            action_result['ip'],
            action_result['host'],
        )
        return output

    def __timeout__(self):
        output = 'timeout {0}\n'.format(self._item)

    @classmethod
    def __format_help__(cls):
        name = cls.__toolname__
        output = '''
Format variables for %s:

 ip       - IP of host
 host     - host of IP
 time     - time consumed by action, in ms

Default format:
 '{ip} {host}'
        ''' % cls.__toolname__
        return output


def main():
    import xnet.tools.run
    xnet.tools.run.run(ResolvTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2011 Krister Hedfors
#

__all__ = ['NetcatsTool']

import unittest
import doctest
import sys
import os
import socket
import fcntl
import gevent
from gevent import select
#from gevent.socket import wait_read

from xnet.tools import Tool
from xnet.net.ipv4 import IPRangeIterator


def _get_output_filepath(item, base):
    #
    # item is the result of some specific processing
    # and may contain arbitrary meta characters
    # which should not affect the file system path
    # where the output file is created.
    #
    item = item.replace('/', '__')
    item = item.replace('\\', '--')
    path = base + item
    return path

class NetcatsTool(Tool):
    '''
        NetcatsTool
            Default format is:
                {addr} {port} [data received from remote host...]

            Input from stdin is sent to all hosts. Each line of
            output from remote host is prefixed by what __format__()
            returns.
    '''

    __toolname__ = 'netcat'
    __itemname__ = 'host'
    __description__ = 'multi-peer netcat-like tool'

    cmdline_options = [
        ('-p', '--port', 'TCP port to connect to',
            dict(dest='port')),
        ('', '--tag-lines', 'tag output lines with target address',
            dict(dest='tag_lines', action='store_true')),
        #('-O', '--open', 'print only results for open ports',
        #    dict(dest='open')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    def __init__(self, options, outfile=sys.stdout, *args, **kw):
        super(NetcatsTool, self).__init__(options, *args, **kw)
        self._outfile = outfile
        self._stdin_pipe = self._nonblocking_pipe()
        self._addr = None
        self._port = 80
        if not self.options.port is None:
            self._port = int(self.options.port)
        self._recv_buffer = ''

    def __del__(self):
        os.close(self._stdin_pipe[0])
        os.close(self._stdin_pipe[1])
        #if not self._outfile in (sys.stdout, sys.stderr):
        #    self._outfile.close()

    def _nonblocking_pipe(self):
        pipe = os.pipe()
        fcntl.fcntl(pipe[0], fcntl.F_SETFL, os.O_NONBLOCK)
        fcntl.fcntl(pipe[1], fcntl.F_SETFL, os.O_NONBLOCK)
        return pipe

    @classmethod
    def __massage__(self, iterator, options):
        for item in iterator:
            item = item.strip()
            try:
                iprangeiter = IPRangeIterator(item)
            except ValueError:
                yield item
            else:
                for ip in iprangeiter:
                    yield str(ip)

    def __parse__(self, item, iterator):
        host = self._host = item.strip()
        addr = (host, self._port)
        self._addr = addr  # for __timeout__()
        result = {'addr': addr}
        return result

    def __action__(self, parse_result):
        addr = parse_result['addr']
        sock = None
        socket_error = None
        state = None
        result = {}
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect(addr)
            state = 'open'
        except socket.error, e:
            #
            # errno numbers differ between platforms,
            # but tests for 'closed' and 'filtered'
            # below should work universally.
            #
            errmsg = e.strerror.lower()
            if 'connection refused' in errmsg:
                state = 'closed'
            elif 'timed out' in errmsg:
                state = 'filtered'
            else:
                state = 'FIXME-' + errmsg.replace(' ', '-')
        #except gevent.GreenletExit:
        #    state = 'filtered'

        outfile = sys.stdout
        if self.options.split_output:
            path = _get_output_filepath(self._host, self.options.split_output)
            outfile = open(path, 'wb')
            fcntl.fcntl(outfile.fileno(), fcntl.F_SETFL, os.O_NONBLOCK)
        self._outfile = outfile

        done = False
        stdin = self._stdin_pipe[0]
        while not done and not sock is None:
            rfds = [stdin, sock]
            rlist, _, _ = select.select(rfds, [], [])
            if stdin in rlist:
                data = os.read(stdin, 4096)
                if len(data) > 0:
                    sock.send(data)
                else:
                    done = True
            if sock in rlist:
                data = sock.recv(4096)
                if len(data) > 0:
                    if self.options.tag_lines:
                        tag = '\n' + self._host + ':'
                        data = tag + data.replace('\n', '\n' + tag)
                    #
                    # Handle random exceptions on file-io
                    # in a non-standard way...
                    #
                    while True:
                        try:
                            self._outfile.write(data)
                        except IOError:
                            pass
                        else:
                            break
                    self._outfile.flush()
                    if self.options.split_tee:
                        sys.stdout.write(data)
                        sys.stdout.flush()
                    #sys.stdout.write(data)
                    #sys.stdout.flush()
                else:
                    done = True

            if 0:
                wait_read(self._stdin_pipe[0])
                data = self._stdin_pipe[0]
                if len(data) > 0:
                    sock.send(data)
                else:
                    done = True
        result['host'] = self._addr[0]
        result['port'] = self._addr[1]
        result['received'] = self._recv_buffer
        result['socket_error'] = socket_error
        return result

    def __format__(self, line, parse_result, action_result):
        output = ''
        if action_result and self.options.verbose:
            output = '{0} {1} {2}'.format(
                action_result['host'],
                action_result['port'],
                action_result['received'],
            )
        return output

    def __timeout__(self):
        if hasattr(self, '_addr'):
            host, port = self._addr
            return 'timeout {0} {1}\n'.format(host, port)
        return None


def main():
    import xnet.tools.run
    xnet.tools.run.run(NetcatsTool, tool_reads_stdin=True)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



# urllib3/filepost.py
# Copyright 2008-2013 Andrey Petrov and contributors (see CONTRIBUTORS.txt)
#
# This module is part of urllib3 and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

import codecs
import mimetypes

from uuid import uuid4
from io import BytesIO

from .packages import six
from .packages.six import b

writer = codecs.lookup('utf-8')[3]


def choose_boundary():
    """
    Our embarassingly-simple replacement for mimetools.choose_boundary.
    """
    return uuid4().hex


def get_content_type(filename):
    return mimetypes.guess_type(filename)[0] or 'application/octet-stream'


def iter_fields(fields):
    """
    Iterate over fields.

    Supports list of (k, v) tuples and dicts.
    """
    if isinstance(fields, dict):
        return ((k, v) for k, v in six.iteritems(fields))

    return ((k, v) for k, v in fields)


def encode_multipart_formdata(fields, boundary=None):
    """
    Encode a dictionary of ``fields`` using the multipart/form-data MIME format.

    :param fields:
        Dictionary of fields or list of (key, value) or (key, value, MIME type)
        field tuples.  The key is treated as the field name, and the value as
        the body of the form-data bytes. If the value is a tuple of two
        elements, then the first element is treated as the filename of the
        form-data section and a suitable MIME type is guessed based on the
        filename. If the value is a tuple of three elements, then the third
        element is treated as an explicit MIME type of the form-data section.

        Field names and filenames must be unicode.

    :param boundary:
        If not specified, then a random boundary will be generated using
        :func:`mimetools.choose_boundary`.
    """
    body = BytesIO()
    if boundary is None:
        boundary = choose_boundary()

    for fieldname, value in iter_fields(fields):
        body.write(b('--%s\r\n' % (boundary)))

        if isinstance(value, tuple):
            if len(value) == 3:
                filename, data, content_type = value
            else:
                filename, data = value
                content_type = get_content_type(filename)
            writer(body).write('Content-Disposition: form-data; name="%s"; '
                               'filename="%s"\r\n' % (fieldname, filename))
            body.write(b('Content-Type: %s\r\n\r\n' %
                       (content_type,)))
        else:
            data = value
            writer(body).write('Content-Disposition: form-data; name="%s"\r\n'
                               % (fieldname))
            body.write(b'\r\n')

        if isinstance(data, int):
            data = str(data)  # Backwards compatibility

        if isinstance(data, six.text_type):
            writer(body).write(data)
        else:
            body.write(data)

        body.write(b'\r\n')

    body.write(b('--%s--\r\n' % (boundary)))

    content_type = str('multipart/form-data; boundary=%s' % boundary)

    return body.getvalue(), content_type



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#

__all__ = [
    'pdb',
    'interactive_debugger_on_exception',
]

import sys

global _orig_excepthook


pdb = None
#
# ipdb prints crap when imported!
#
try:
    mypdb = __import__('ipdb')
except ImportError:
    mypdb = __import__('pdb')
pdb = mypdb


def _stacktrace_and_pm(type, value, tb):
    import traceback
    traceback.print_exception(type, value, tb)
    pdb.pm()


_orig_excepthook = None


def interactive_debugger_on_exception(active):
    global _orig_excepthook
    if _orig_excepthook is None:
        _orig_excepthook = sys.excepthook
    if active:
        sys.excepthook = _stacktrace_and_pm
    else:
        assert not _orig_excepthook is None
        sys.excepthook = _orig_excepthook


def _test():
    import doctest
    import unittest
    doctest.testmod()
    unittest.main()


if __name__ == "__main__":
    _test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#

from xnet.tests.test_ipv4 import *


def _test():
    import doctest
    import unittest
    doctest.testmod()
    unittest.main()

if __name__ == "__main__":
    _test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2011 Krister Hedfors
#

__all__ = ['TcpStateTool']


import socket

from xnet.tools import Tool
from xnet.tools import ToolMixin_AddrIterator


class TcpStateTool(ToolMixin_AddrIterator, Tool):
    '''
        TcpStateTool
            Default format is:
                {addr} {port} {state}
            Output is sent to stderr when state
            is neither 'open', 'closed' nor 'filtered',
            (such as address resolution errors).
    '''

    __toolname__ = 'tcpstate'
    __itemname__ = 'host'
    __description__ = 'test if TCP-ports are open'

    DEFAULT_PORT = 80

    cmdline_options = [
        ('-p', '--port', 'port to state check',
            dict(dest='port')),
        ('-O', '--open', 'only print open ports',
            dict(dest='open', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    def __repr__(self):
        state = ('x', 'finished')[self.finished]
        if hasattr(self, '_addr'):
            host, port = self._addr
            return '<tcpstate {0}:{1} {2}>'.format(host, port, state)
        return '<tcpstate {0}>'.format(state)

    def __action__(self, parse_result):
        addr = parse_result['addr']
        state = None
        result = {}
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(addr)
            state = 'open'
        except socket.error, e:
            #
            # errno numbers differ between platforms,
            # but tests for 'closed' and 'filtered'
            # below should work universally.
            #
            errmsg = e.strerror.lower()
            if 'connection refused' in errmsg:
                state = 'closed'
            elif 'timed out' in errmsg:
                state = 'filtered'
            else:
                state = errmsg.replace(' ', '-').replace(',', '-')
        #except gevent.GreenletExit:
        #    #
        #    # happens in BT, not OSX
        #    #
        #    state = 'filtered'
        finally:
            if s:
                s.close()
        result['host'] = addr[0]
        result['port'] = addr[1]
        result['state'] = state
        return result

    def __filter__(self, action_result):
        if self.options.open:
            return action_result and action_result['state'] == 'open'
        return action_result

    def __format__(self, line, parse_result, action_result):
        output = ''
        if action_result:
            if self.options.open:
                output = '{0} {1}'.format(
                    action_result['host'],
                    action_result['port'],
                )
            else:
                output = '{0} {1} {2}'.format(
                    action_result['host'],
                    action_result['port'],
                    action_result['state'],
                )
        return output

    @classmethod
    def __format_help__(cls):
        output = '''
Format variables for %s:

 host    - target host
 port    - target port
 state   - TCP-state or error-message
 time    - time consumed by action, in ms

Default format:
 '{host} {port} {state}'
        ''' % cls.__toolname__
        return output

    def __timeout__(self):
        if hasattr(self, '_addr'):
            host, port = self._addr
            return '{0} {1} wait-timeout\n'.format(host, port)
        return None


def main():
    import xnet.tools.run
    xnet.tools.run.run(TcpStateTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#
# iterera ranges och spara varje output
# i set foer att stoppa dubletter.
# Skit i performance.


import unittest
import doctest

from xnet.tools import Tool
from xnet.net.ipv4 import IPRangeIterator


class IPRangeTool(Tool):

    __description__ = 'generate list of IPs from IP ranges'
    __itemname__ = 'iprange'

    cmdline_options = [
        ('', '--print-duplicates', 'stateless, faster',
            dict(dest='print_duplicates', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    def __init__(self, *args, **kw):
        super(IPRangeTool, self).__init__(*args, **kw)
        #
        # Use shared state of printed IPs between all
        # IPRangeTool instances.
        #
        if not self.options.print_duplicates and \
                not hasattr(self.__class__, '_iprange_state'):
            self.__class__._iprange_state = set()

    def __parse__(self, line, iterator):
        iprange = line.strip()
        return {'iprange': iprange}

    def __action__(self, parse_result):
        iprange = parse_result['iprange']
        iprangeiter = None
        if not self.options.print_duplicates:
            iprangeiter = IPRangeIterator(iprange, self._get_iprange_state())
        else:
            iprangeiter = IPRangeIterator(iprange)
        return {'iprangeiter': iprangeiter}

    def _get_iprange_state(self):
        return self.__class__._iprange_state

    def _set_iprange_state(self, iprange_state):
        self.__class__._iprange_state = iprange_state

    def __format__(self, line, parse_result, action_result):
        output = ''
        iprangeiter = action_result['iprangeiter']
        iplist = list(iprangeiter)
        output += '\n'.join(iplist)
        return output

    def __format_help__(self):
        return 'No customized formatting available\n'


def main():
    import xnet.tools.run
    xnet.tools.run.run(IPRangeTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#


def raises(exc, fn, *args, **kw):
    '''
        doesn't catch any exceptions other than exc

        >>> raises(ValueError, int, 'foo')
        True
        >>> raises(ValueError, int, '1')
        False
    '''
    try:
        fn(*args, **kw)
    except exc:
        return True
    return False


def assert_raises(exc, fn, *args, **kw):
    '''
        returns nothing if expected exception is raised,
        raises AssertionError if no exception in raised,
        leave all other exceptions unhandled

        >>> assert_raises(ValueError, int, 'foo')
        >>> raises(AssertionError, assert_raises, Exception, lambda: 1)
        True
    '''
    try:
        fn(*args, **kw)
    except exc:
        return
    raise AssertionError


def _test():
    import doctest
    import unittest
    doctest.testmod()
    unittest.main()

if __name__ == "__main__":
    _test()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#

import gevent
from xnet.tools import Tool


class DummyTool(Tool):

    __description__ = 'dummy to use for testing'
    __itemname__ = 'dummy'

    cmdline_options = [
        ('', '--sleep', 'gevent.sleep() for each input, treat items as float',
            dict(dest='sleep', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    def __parse__(self, line, iterator):
        item = line.strip()
        return {'item': item}

    def __action__(self, parse_result):
        item = parse_result['item']
        result = {}
        if self.options.sleep:
            sleeptime = float(item)
            gevent.sleep(sleeptime)
            result = {'sleeptime': sleeptime}
        return result

    def __format__(self, line, parse_result, action_result):
        output = ''
        if self.options.sleep:
            sleeptime = action_result['sleeptime']
            output += 'slept:{0} '.format(sleeptime)
        return output


def main():
    import xnet.tools.run
    xnet.tools.run.run(DummyTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#

import re


class IPException(Exception):
    pass


class IP(object):
    '''
        >>> from xnet.testing.exceptions import assert_raises
        >>> IP('1.2.3.4').__str__()
        '1.2.3.4'

        >>> assert_raises(ValueError, lambda: IP(''))
        >>> assert_raises(ValueError, lambda: IP('foo'))
        >>> assert_raises(ValueError, lambda: IP('1.2'))
        >>> assert_raises(ValueError, lambda: IP('1.2.3'))
        >>> assert_raises(ValueError, lambda: IP('1.2.3.4.5'))
        >>> assert_raises(ValueError, lambda: IP(None))

        >>> assert int(IP('0.0.0.1')) == 0x00000001 == 1
        >>> assert int(IP('0.0.1.0')) == 0x00000100 == 256
        >>> assert int(IP('0.1.0.0')) == 0x00010000 == 65536
        >>> assert int(IP('1.0.0.0')) == 0x01000000 == 16777216

        >>> assert IP(1).__str__() == '0.0.0.1'
        >>> assert IP(256).__str__() == '0.0.1.0'
        >>> assert IP(65536).__str__() == '0.1.0.0'
        >>> assert IP(16777216).__str__() == '1.0.0.0'
    '''

    _E_INVALID = 'Invalid IP address: {0}'

    def __init__(self, ip):
        self._octets = self._parse_ip(ip)

    def __str__(self):
        return '{0}.{1}.{2}.{3}'.format(*self._octets)

    def __repr__(self):
        return "<IP('{0}')>".format(self.__str__())

    def _parse_ip(self, ip):
        if isinstance(ip, self.__class__):
            return ip._octets
        ip = str(ip)
        if ip.isdigit():
            ip = int(ip)
            return self._parse_ip_int(ip)
        if not '.' in ip:
            errmsg = self._E_INVALID.format(ip)
            raise ValueError(errmsg)
        w = ip.split('.')
        if not len(w) == 4:
            errmsg = self._E_INVALID.format(ip)
            raise ValueError(errmsg)
        try:
            octets = map(int, w)
        except ValueError:
            errmsg = self._E_INVALID.format(ip)
            raise ValueError(errmsg)
        for octet in octets:
            if not (0 <= octet <= 255):
                errmsg = self._E_INVALID.format(ip)
                raise ValueError(errmsg)
        octets = tuple(octets)
        return octets

    def _parse_ip_int(self, n):
        a = (n >> 24) & 0xff
        b = (n >> 16) & 0xff
        c = (n >> 8) & 0xff
        d = (n >> 0) & 0xff
        return (a, b, c, d)

    def __int__(self):
        n0, n1, n2, n3 = self._octets
        n = (n0 << 24) + (n1 << 16) + (n2 << 8) + n3
        return n


class Netmask(object):
    '''
        >>> netmask = Netmask('255.255.255.0')
        >>> netmask.get_network_part('1.2.3.4')
        <IP('1.2.3.0')>
        >>> netmask.get_host_part('1.2.3.4')
        <IP('0.0.0.4')>
    '''

    _E_INVALID = 'Invalid netmask: {0}'

    _bitlen_to_bitmask = dict([
        (i, 0xffffffffL & (0xffffffffL << (32 - i))) for i in xrange(0, 33)
    ])

    _bitmask_to_bitlen = dict([
        (0xffffffffL & (0xffffffffL << (32 - i)), i) for i in xrange(0, 33)
    ])

    def __init__(self, netmask):
        (self._bitlen, self._bitmask) = self._parse_netmask(netmask)

    def _parse_netmask(self, netmask):
        bitlen = None
        bitmask = None
        if isinstance(netmask, self.__class__):
            return (netmask._bitlen, netmask._bitmask)
        #
        # short netmask syntax
        #
        if netmask.isdigit():
            bitlen = int(netmask)
            if not (0 <= bitlen <= 32):
                raise ValueError(self._E_INVALID.format(netmask))
            bitmask = self._bitlen_to_bitmask[bitlen]
            return (bitlen, bitmask)
        #
        # long netmask syntax
        #
        try:
            addr = IP(netmask)
        except ValueError:
            raise ValueError(self._E_INVALID.format(netmask))
        bitmask = int(addr)
        if not bitmask in self._bitmask_to_bitlen:
            raise ValueError(self._E_INVALID.format(netmask))
        bitlen = self._bitmask_to_bitlen[bitmask]
        return (bitlen, bitmask)

    @property
    def bitlen(self):
        return self._bitlen

    @property
    def bitmask(self):
        return self._bitmask

    def get_first_addr(self, ip):
        return self.get_network_part(ip)

    def get_last_addr(self, ip):
        host_part = self._bitmask ^ 0xffffffff
        addr = int(self.get_network_part(ip)) + host_part
        return IP(addr)

    def get_network_part(self, ip):
        if not isinstance(ip, IP):
            ip = IP(ip)
        network = int(ip) & self._bitmask
        return IP(network)

    def get_host_part(self, ip):
        if not isinstance(ip, IP):
            ip = IP(ip)
        host = int(ip) & ~self._bitmask
        return IP(host)


class IPRangeIterator(object):
    '''
        >>> ipr = IPRangeIterator('1.2.3.4')
        >>> assert ipr._is_ip('1.2.3.4')
        >>> assert not ipr._is_ip('1.2.3.x')

        >>> ipr = IPRangeIterator('1.2.3.*')
        >>> assert '1.2.3.4' in ipr
        >>> assert '1.2.3.9' in ipr
        >>> assert not '1.2.9.4' in ipr
        >>> assert not '1.9.3.4' in ipr
        >>> assert not '9.2.3.4' in ipr

        >>> ipr = IPRangeIterator('5-6.1.1.1')
        >>> assert not '4.1.1.1' in ipr
        >>> assert '5.1.1.1' in ipr
        >>> assert '6.1.1.1' in ipr
        >>> assert not '7.1.1.1' in ipr
    '''

    _E_INVALID = 'Invalid IP-range: {0}'
    _E_NO_NETMASK = 'Netmasks not allowed here: {0}'

    def __init__(self, iprange, state=None, allow_netmasks=True):
        self._iprange = iprange
        self._state = state
        self._ip = None
        self._netmask = None
        self._octet_ranges = None
        #
        iprange = str(iprange)
        #
        # If netmask, accept only IP and
        # iterate all hosts within that network.
        # Example:  1.2.3.0/24
        #
        if '/' in iprange:
            if not allow_netmasks:
                raise ValueError(self._E_NO_NETMASK.format(iprange))
            tmp = iprange.split('/')
            if not len(tmp) == 2:
                raise ValueError(self._E_INVALID.format(iprange))
            self._ip = IP(tmp[0])
            self._netmask = Netmask(tmp[1])
        #
        # If no netmask, handle range and wildcard
        # notations as usual.
        # Examples: 1.2.3.*, 10.0.0-5.*
        #
        else:
            if not '.' in iprange or iprange.split('.').__len__() != 4:
                raise ValueError(self._E_INVALID.format(iprange))
            w = iprange.split('.')
            octet_ranges = map(self._get_octet_range, w)
            self._octet_ranges = octet_ranges

    #
    # broken, never invoked
    #
    def __in__(self, val):
        int_octets = self._get_ip_octets(val)
        for i in xrange(4):
            octet = int_octets[i]
            (lower, upper) = self._octet_ranges[i]
            if not (lower <= octet < upper):
                return False
        return True

    def _is_ip(self, val):
        expr = re.compile(r'^(\d+)\.(\d+)\.(\d+)\.(\d+)$')
        m = expr.match(val)
        if not m:
            return False
        octets = [m.group(i) for i in xrange(1, 5)]
        int_octets = map(lambda d: int(d), octets)
        if False in map(lambda n: (0 <= n <= 255), int_octets):
            return False
        return True

    def _get_ip_octets(self, ipstring):
        expr = re.compile(r'^(\d+)\.(\d+)\.(\d+)\.(\d+)$')
        m = expr.match(ipstring)
        if not m:
            return False
        octets = [m.group(i) for i in xrange(1, 5)]
        int_octets = map(lambda d: int(d), octets)
        if False in map(lambda n: (0 <= n <= 255), int_octets):
            return False
        return int_octets

    def _get_octet_range(self, octet):
        if octet == '*':
            return (0, 256)
        elif octet.isdigit():
            return (int(octet), int(octet) + 1)
        elif '-' in octet:
            (start, end) = octet.split('-', 1)
            if start.isdigit() and end.isdigit():
                return (int(start), int(end) + 1)  # ranges are inclusive
        raise ValueError(self._E_INVALID.format(self._iprange))

    def __iter__(self):
        if not self._octet_ranges is None:
            return self._generate_from_octet_ranges()
        elif not self._netmask is None:
            return self._generate_from_netmask()
        assert not 'Logic error'

    def _generate_from_octet_ranges(self):
        A, B, C, D = self._octet_ranges
        state = self._state
        for a in xrange(*A):
            for b in xrange(*B):
                for c in xrange(*C):
                    for d in xrange(*D):
                        ip = '{0}.{1}.{2}.{3}'.format(a, b, c, d)
                        if not state is None:
                            if ip in state:
                                continue
                            state.add(ip)
                        yield ip

    def _generate_from_netmask(self):
        start = self._netmask.get_first_addr(self._ip).__int__()
        end = self._netmask.get_last_addr(self._ip).__int__()
        i = start
        while i <= end:
            yield IP(i).__str__()
            i += 1


class PortRangeIterator(object):
    '''
        >>> assert [5] == list(PortRangeIterator('5'))
        >>> assert [5,6,7] == list(PortRangeIterator('5-7'))
        >>> assert [1,2,8,9] == list(PortRangeIterator('1-2,8-9'))
        >>> assert [1,2,5,8,9] == list(PortRangeIterator('1-2,5,8-9'))
        >>> assert [1,2,1,2,1,2] == list(PortRangeIterator('1-2,1,2,1-2'))
    '''

    _E_INVALID = 'Invalid port-range: {0}'

    def __init__(self, portrange):
        self._portrange = portrange

    def __iter__(self):
        values = self._portrange.split(',')
        for value in values:
            (start, end) = self._get_value_range(value)
            for port in xrange(start, end):
                yield port

    def _get_value_range(self, value):
        if value.isdigit():
            return (int(value), int(value) + 1)
        elif '-' in value:
            (start, end) = value.split('-', 1)
            if start.isdigit() and end.isdigit():
                if start <= end:
                    return (int(start), int(end) + 1)  # ranges are inclusive
        raise ValueError(self._E_INVALID.format(self._portrange))


class AddressIterator:
    '''
        >>> ai = lambda s: list(AddressIterator().expand(s))
        >>> assert ai('10.0.0.0') == ['10.0.0.0:80']
        >>> assert ai('10.0.0.1-3') == ['10.0.0.1:80', '10.0.0.2:80', '10.0.0.3:80']
        >>> assert ai('10.0.0.1-3:5') == ['10.0.0.1:5', '10.0.0.2:5', '10.0.0.3:5']
        >>> assert ai('3-4.0.0.0:5,6') == ['3.0.0.0:5', '3.0.0.0:6', '4.0.0.0:5', '4.0.0.0:6']
    '''
    def __init__(self, default_ports=[80], **iprange_kwargs):
        '''
            default_ports - iterable or str-type port range
        '''
        if type(default_ports) is str:
            default_ports = list(PortRangeIterator(default_ports))
        self._default_ports = default_ports
        self._iprange_kwargs = iprange_kwargs

    def expand(self, addr):
        addr = str(addr).strip()
        hostpart = ''
        portpart = ''
        hosts = []
        ports = []
        #
        if ':' in addr:
            hostpart, portpart = addr.split(':', 1)
            ports = PortRangeIterator(portpart)
        else:
            hostpart = addr
            ports = self._default_ports

        try:
            hosts = list(IPRangeIterator(hostpart, **self._iprange_kwargs))
        except ValueError:
            hosts = [hostpart]

        for host in hosts:
            for port in ports:
                yield '{0}:{1}'.format(host, port)


def _test():
    import doctest
    import unittest
    print('doctest: ' + doctest.testmod().__str__())
    unittest.main()

if __name__ == "__main__":
    _test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2011 Krister Hedfors
#

__all__ = ['NetfuzzTool']


import socket
import urllib
import base64

from xnet.tools import Tool


def _tcp4_connect(addr):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect(addr)
    return sock


class Template(object):
    def __init__(self, path):
        f = open(path, 'rb')
        self._template = f.read()
        f.close()
        self._slices = self._template.split('$$')
        self._nr_placeholders = len(self._slices) - 1

    def sniper(self, payload):
        s = self._slices
        for i in xrange(1, len(s)):
            head = ''.join(s[:i])
            foot = ''.join(s[i:])
            result = head + payload + foot
            yield result


class NetfuzzTool(Tool):
    '''
        NetfuzzTool
    '''

    __toolname__ = 'netfuzz'
    __itemname__ = 'host'
    __description__ = 'Send various payloads to target'

    cmdline_options = [
        ('-a', '--address', 'Target address, specified as host:port',
            dict(dest='address', metavar='addr')),
        ('-t', '--template', 'Template containing $$ placeholders where payloads are inserted',
            dict(dest='template', metavar='tmpl')),
        ('', '--tag-lines', 'tag output lines with insertion point index and payload',
            dict(dest='tag_lines', action='store_true')),
        ('-u', '--urlenc', 'urlencode payloads',
            dict(dest='urlenc', action='store_true')),
        ('-b', '--base64', 'base64-encode payloads',
            dict(dest='base64', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    def _new_connection(self):
        return _tcp4_connect(self._addr)

    @classmethod
    def __setup__(cls, options):
        cls._template = Template(options.template)
        address = options.address
        assert ':' in address
        (host, port) = address.split(':')
        assert port.isdigit()
        cls._addr = (host, int(port))
        conn = _tcp4_connect(cls._addr)  # verify connectivity
        conn.close()

    #
    # NOTE! passing dict from __massage__ to __parse__
    # in order to preserve payload info for instances.
    #
    @classmethod
    def __massage__(cls, iterator, options):
        for (pindex, payload) in enumerate(iterator):
            if payload[-1] == '\n':
                payload = payload[:-1]
            if options.base64:
                payload = base64.b64encode(payload)
            if options.urlenc:
                payload = urllib.quote(payload)
            for (inspoint, request) in enumerate(cls._template.sniper(payload)):
                yield {
                    'request': request,
                    'payload': payload,
                    'pindex': pindex,
                    'inspoint': inspoint,
                }

    def __parse__(self, item, iterator):
        return item

    def __action__(self, parse_result):
        request = parse_result['request']
        result = parse_result.copy()
        conn = None
        #
        conn = self._new_connection()
        conn.send(request)
        response = conn.recv(65535)
        if response and self.options.tag_lines:
            tag = '{0},{1}:'.format(result['inspoint'], result['payload'])
            response = '\n' + tag + response.replace('\n', '\n' + tag)
        result['addr'] = '{0}:{1}'.format(*self._addr)
        result['host'] = self._addr[0]
        result['port'] = self._addr[1]
        result['response'] = response
        return result

    def __filter__(self, action_result):
        return action_result

    def __format__(self, line, parse_result, action_result):
        return action_result['response']

    @classmethod
    def __format_help__(cls):
        output = '''
Format variables for %s:

 addr     - target address
 host     - target host
 port     - target port
 response - server response
 request  - client request
 payload  - payload
 pindex   - payload index
 inspoint - insertion point, starting from 0
 time     - time consumed by action, in ms

Default format:
 '{response}'
        ''' % cls.__toolname__
        return output

    def __timeout__(self):
        if hasattr(self, '_addr'):
            return '{0}:{1} wait-timeout\n'.format(*self._addr)
        return None

    def __filename__(self):
        return 'pindex_{0}_inspoint_{1}'.format(
            self._action_result['pindex'],
            self._action_result['inspoint'],
        )


def main():
    import xnet.tools.run
    xnet.tools.run.run(NetfuzzTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



# urllib3/response.py
# Copyright 2008-2013 Andrey Petrov and contributors (see CONTRIBUTORS.txt)
#
# This module is part of urllib3 and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php


import logging
import zlib

from .exceptions import DecodeError
from .packages.six import string_types as basestring, binary_type


log = logging.getLogger(__name__)


class DeflateDecoder(object):

    def __init__(self):
        self._first_try = True
        self._data = binary_type()
        self._obj = zlib.decompressobj()

    def __getattr__(self, name):
        return getattr(self._obj, name)

    def decompress(self, data):
        if not self._first_try:
            return self._obj.decompress(data)

        self._data += data
        try:
            return self._obj.decompress(data)
        except zlib.error:
            self._first_try = False
            self._obj = zlib.decompressobj(-zlib.MAX_WBITS)
            try:
                return self.decompress(self._data)
            finally:
                self._data = None


def _get_decoder(mode):
    if mode == 'gzip':
        return zlib.decompressobj(16 + zlib.MAX_WBITS)

    return DeflateDecoder()


class HTTPResponse(object):
    """
    HTTP Response container.

    Backwards-compatible to httplib's HTTPResponse but the response ``body`` is
    loaded and decoded on-demand when the ``data`` property is accessed.

    Extra parameters for behaviour not present in httplib.HTTPResponse:

    :param preload_content:
        If True, the response's body will be preloaded during construction.

    :param decode_content:
        If True, attempts to decode specific content-encoding's based on headers
        (like 'gzip' and 'deflate') will be skipped and raw data will be used
        instead.

    :param original_response:
        When this HTTPResponse wrapper is generated from an httplib.HTTPResponse
        object, it's convenient to include the original for debug purposes. It's
        otherwise unused.
    """

    CONTENT_DECODERS = ['gzip', 'deflate']

    def __init__(self, body='', headers=None, status=0, version=0, reason=None,
                 strict=0, preload_content=True, decode_content=True,
                 original_response=None, pool=None, connection=None):
        self.headers = headers or {}
        self.status = status
        self.version = version
        self.reason = reason
        self.strict = strict
        self.decode_content = decode_content

        self._decoder = None
        self._body = body if body and isinstance(body, basestring) else None
        self._fp = None
        self._original_response = original_response

        self._pool = pool
        self._connection = connection

        if hasattr(body, 'read'):
            self._fp = body

        if preload_content and not self._body:
            self._body = self.read(decode_content=decode_content)

    def get_redirect_location(self):
        """
        Should we redirect and where to?

        :returns: Truthy redirect location string if we got a redirect status
            code and valid location. ``None`` if redirect status and no
            location. ``False`` if not a redirect status code.
        """
        if self.status in [301, 302, 303, 307]:
            return self.headers.get('location')

        return False

    def release_conn(self):
        if not self._pool or not self._connection:
            return

        self._pool._put_conn(self._connection)
        self._connection = None

    @property
    def data(self):
        # For backwords-compat with earlier urllib3 0.4 and earlier.
        if self._body:
            return self._body

        if self._fp:
            return self.read(cache_content=True)

    def read(self, amt=None, decode_content=None, cache_content=False):
        """
        Similar to :meth:`httplib.HTTPResponse.read`, but with two additional
        parameters: ``decode_content`` and ``cache_content``.

        :param amt:
            How much of the content to read. If specified, caching is skipped
            because it doesn't make sense to cache partial content as the full
            response.

        :param decode_content:
            If True, will attempt to decode the body based on the
            'content-encoding' header.

        :param cache_content:
            If True, will save the returned data such that the same result is
            returned despite of the state of the underlying file object. This
            is useful if you want the ``.data`` property to continue working
            after having ``.read()`` the file object. (Overridden if ``amt`` is
            set.)
        """
        # Note: content-encoding value should be case-insensitive, per RFC 2616
        # Section 3.5
        content_encoding = self.headers.get('content-encoding', '').lower()
        if self._decoder is None:
            if content_encoding in self.CONTENT_DECODERS:
                self._decoder = _get_decoder(content_encoding)
        if decode_content is None:
            decode_content = self.decode_content

        if self._fp is None:
            return

        flush_decoder = False

        try:
            if amt is None:
                # cStringIO doesn't like amt=None
                data = self._fp.read()
                flush_decoder = True
            else:
                cache_content = False
                data = self._fp.read(amt)
                if amt != 0 and not data:  # Platform-specific: Buggy versions of Python.
                    # Close the connection when no data is returned
                    #
                    # This is redundant to what httplib/http.client _should_
                    # already do.  However, versions of python released before
                    # December 15, 2012 (http://bugs.python.org/issue16298) do not
                    # properly close the connection in all cases. There is no harm
                    # in redundantly calling close.
                    self._fp.close()
                    flush_decoder = True

            try:
                if decode_content and self._decoder:
                    data = self._decoder.decompress(data)
            except (IOError, zlib.error):
                raise DecodeError("Received response with content-encoding: %s, but "
                                  "failed to decode it." % content_encoding)

            if flush_decoder and self._decoder:
                buf = self._decoder.decompress(binary_type())
                data += buf + self._decoder.flush()

            if cache_content:
                self._body = data

            return data

        finally:
            if self._original_response and self._original_response.isclosed():
                self.release_conn()

    @classmethod
    def from_httplib(ResponseCls, r, **response_kw):
        """
        Given an :class:`httplib.HTTPResponse` instance ``r``, return a
        corresponding :class:`urllib3.response.HTTPResponse` object.

        Remaining parameters are passed to the HTTPResponse constructor, along
        with ``original_response=r``.
        """

        # Normalize headers between different versions of Python
        headers = {}
        for k, v in r.getheaders():
            # Python 3: Header keys are returned capitalised
            k = k.lower()

            has_value = headers.get(k)
            if has_value: # Python 3: Repeating header keys are unmerged.
                v = ', '.join([has_value, v])

            headers[k] = v

        # HTTPResponse objects in Python 3 don't have a .strict attribute
        strict = getattr(r, 'strict', 0)
        return ResponseCls(body=r,
                           headers=headers,
                           status=r.status,
                           version=r.version,
                           reason=r.reason,
                           strict=strict,
                           original_response=r,
                           **response_kw)

    # Backwards-compatibility methods for httplib.HTTPResponse
    def getheaders(self):
        return self.headers

    def getheader(self, name, default=None):
        return self.headers.get(name, default)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#

__all__ = (
    'Test_Netmask',
    'Test_IPRangeIterator',
)


import doctest
import unittest

from xnet.net.ipv4 import (
    IP,
    Netmask,
    IPRangeIterator,
)


class Test_Netmask(unittest.TestCase):

    def test_basics(self):
        netmask = Netmask('255.255.255.0')
        self.assertEqual(netmask.bitmask, 0xffffff00)
        self.assertEqual(netmask.bitlen, 24)
        self.assertEqual(
            netmask.get_first_addr('1.2.3.4').__str__(),
            '1.2.3.0')
        self.assertEqual(
            netmask.get_last_addr('1.2.3.4').__str__(),
            '1.2.3.255')


class Test_IPRangeIterator(unittest.TestCase):

    def test_netmask_short_notation(self):
        for i in xrange(22, 32):  # cant run all, to expensive
            it = IPRangeIterator('10.0.0.0/{0}'.format(i))
            lst = list(it)
            #
            # validate number of returned IPs
            #
            assert len(lst) == 2 ** (32 - i)
            prev = IP(0)
            #
            # validate incrementating aspect of range
            #
            for ipnum in lst:
                ip = IP(ipnum)
                assert int(prev) < int(ip)
                prev = ip


def _test():
    print(doctest.testmod())
    unittest.main()

if __name__ == "__main__":
    _test()



#!/usr/bin/env python

# Bootstrap installation of Distribute
#import distribute_setup
#distribute_setup.use_setuptools()

import os

from setuptools import setup


PROJECT = 'xnet'
VERSION = '0.2.1.0'  # release 0.2.1.0 when all tools support expandable addrs
URL = 'http://xnet.tripleaes.com/'
AUTHOR = u'Krister Hedfors'
AUTHOR_EMAIL = u'krister.hedfors@gmail.com'
DESC = "XNet - Just like Xmas, but on your networks."


def read_file(file_name):
    file_path = os.path.join(
        os.path.dirname(__file__),
        file_name
    )
    return open(file_path).read()

setup(
    name=PROJECT,
    version=VERSION,
    description=DESC,
    long_description=read_file('README.rst'),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license='LICENSE',
    #py_module=['xnet'],
    #namespace_packages=[],
    packages=[
        'xnet',
        'xnet.tests',
        'xnet.testing',
        'xnet.debug',
        'xnet.tools',
        'xnet.net',
        'xnet.net.ipv4',
        'xnet.packages',
        'xnet.packages.urllib3',
        'xnet.packages.urllib3.packages',
        'xnet.packages.urllib3.packages.ssl_match_hostname',
    ],
    #package_dir = {{'': os.path.dirname(__file__)}},
    #package_dir = {'xnet': 'xnet'},
    #include_package_data=True,
    #zip_safe=False,
    #install_requires=[
    #    # -*- Requirements -*-
    #],
    entry_points={
        #
        # populate automatically
        #
        'console_scripts': [
            'xnet = xnet.tools.run:main',
            'iissn = xnet.tools.iissn:main',
            'iprange = xnet.tools.iprange:main',
            'resolv = xnet.tools.resolv:main',
            'sslinfo = xnet.tools.sslinfo:main',
            'tcpstate = xnet.tools.tcpstate:main',
            'webget = xnet.tools.webget:main',
            'netcats = xnet.tools.netcats:main',
        ],
    },
    classifiers=[
        # see http://pypi.python.org/pypi?:action=list_classifiers
        # -*- Classifiers -*-
        'License :: OSI Approved',
        'License :: OSI Approved :: BSD License',
        "Programming Language :: Python",
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#
# Known bugs:
#   Debian Squeeze, apt-get:ed python-gevent
#       * some DNS queries for nonexistent hosts
#         are not cancelled by -w, but takes a long time to complete.
#


__all__ = ['WebgetTool']


#import unittest
#import doctest
import sys
import os
import codecs
import urlparse

from xnet.tools import Tool
from xnet.net.ipv4 import PortRangeIterator
from xnet.net.http import UrlIterator
#from xnet.tools import ToolMixin_AddrIterator
#from xnet.debug import pdb
import xnet.packages.urllib3
import xnet.packages.urllib3.exceptions


class WebgetException(Exception):
    pass


class WebgetTool(Tool):

    __toolname__ = 'webget'
    __itemname__ = 'url'
    __description__ = 'get information from web services'

    cmdline_options = [
        ('-p', '--ports', 'use these ports for urls lacking port AND protocol components',
            dict(dest='ports', metavar='m-n,..')),
        ('-s', '--server', 'print server header',
            dict(dest='server', action='store_true')),
        ('-S', '--https', 'default to https when protocol is missing',
            dict(dest='https', action='store_true')),
        ('', '--host', 'specify Host header manually',
            dict(dest='host')),
        ('', '--method', 'specify alternative method. Examples GET, HEAD, POST, ...',
            dict(dest='method', metavar='method')),
        ('', '--proxy', 'broken, sorry!',
            dict(dest='proxy')),
        ('', '--code', 'only print response code',
            dict(dest='code', action='store_true')),
        ('', '--redirect-allow-samedomain', 'follow redirects under the same domain',
            dict(dest='redirect_allow_samedomain', action='store_true')),
        ('', '--redirect-whitelist', 'comma-separated whitelist of allowed redirect hosts',
            dict(dest='redirect_whitelist', metavar='hosts')),
        ('', '--xpath', 'grab elements matching xpath expression',
            dict(action='append', metavar='expr')),
        ('', '--xpath-delimiter', 'xpath output delimiter when using multiple --xpath\'s',
            dict(dest='xpath_delimiter', metavar='delim')),
        ('', '--xpath-strict', 'exit if not all --xpath\'s are matched',
            dict(dest='xpath_strict', action='store_true')),
        ('', '--tag-lines', 'prefix each line with current url',
            dict(dest='tag_lines', action='store_true')),
    ]

    @classmethod
    def print_source(cls):
        cls._print_source(__file__)

    MAX_REDIRECTS = 8

    def __init__(self, *args, **kw):
        super(WebgetTool, self).__init__(*args, **kw)
        if self.options.proxy:
            msg = '--proxy is broken atm, sorry!'
            raise Exception(msg)
        if not hasattr(self, '_http'):
            self._http = self.__class__._http = self._get_connection()
        self._url = None
        self._redirect_whitelist = set()
        if self.options.redirect_whitelist:
            allowed_hosts = self.options.redirect_whitelist.split(',')
            allowed_hosts = [h.lower() for h in allowed_hosts]
            self._redirect_whitelist = set(allowed_hosts)

    def _get_connection(self):
        if self.options.proxy:
            return xnet.packages.urllib3.proxy_from_url(self.options.proxy)
        else:
            return xnet.packages.urllib3.PoolManager()

    def _request(self, *args, **kw):
        result = None
        done = False
        while not done:
            try:
                result = self._http.request(*args, **kw)
            except xnet.packages.urllib3.exceptions.ClosedPoolError:
                pass  # urllib3 seems to take care of this
            else:
                done = True
        return result

    @classmethod
    def __massage__(cls, iterator, options):
        default_scheme = 'http'
        default_ports = [80]
        if options.https:
            default_scheme = 'https'
            default_ports = [443]
        if options.ports:
            default_ports = list(PortRangeIterator(options.ports))
        urlIterator = UrlIterator(default_scheme=default_scheme, default_ports=default_ports)
        for xurl in iterator:
            for url in urlIterator.expand(xurl):
                yield url

    def __parse__(self, url, iterator):
        url = url.strip()
        result = {}
        result['url'] = url
        result['host'] = self._get_host(url)
        return result

    def _explicitly_allowed_redirect(self, ourl, lurl):
        if lurl.netloc.lower() in self._redirect_whitelist:
            return True
        if self.options.redirect_allow_samedomain:
            if not lurl.scheme and not lurl.netloc:
                return True
            ow = ourl.netloc.lower().split('.')
            lw = lurl.netloc.lower().split('.')
            if len(ow) > 1 and len(lw) > 1:
                if not ow[-1].isdigit():
                    if ow[-2:] == lw[-2:]:
                        return True
        return False

    def _get_host(self, url):
        return urlparse.urlparse(url).netloc

    def __action__(self, parse_result):
        result = {}
        _url = self._url = parse_result['url']
        _host = self.options.host or self._get_host(_url)
        _method = self.options.method or 'GET'
        _response = None
        _body_unicode = None
        _code = None
        _msg = None
        _body = ''
        #
        #proxies = {}
        #
        request_kwargs = {}
        request_kwargs['redirect'] = False

        if self.options.host:
            request_kwargs['headers'] = {'Host': self.options.host}
            _host = self.options.host

        read_response = not self.options.code \
            and not self.options.server

        if self.options.format and '{body}' in self.options.format:
            read_response = True

        done = False
        redirect_count = 0
        #
        # loop to handle redirects
        #
        while not done:
            try:
                _response = self._request(_method, _url, **request_kwargs)
            except xnet.tools.WaitTimeout:
                raise
            except xnet.packages.urllib3.exceptions.MaxRetryError, e:
                if e.reason and e.reason.strerror:
                    self.set_error(e.reason.strerror, oneword=True)
                else:
                    self.set_error(e)
                return None
            except:
                import traceback
                print  traceback.print_exc(sys.exc_info()[1])
                e = str(sys.exc_info())
                self.set_error(e)
                self.stderr = True
                return None

            _code = _response.status
            _msg = _response.reason

            if not _code in [301, 302, 303, 307]:
                done = True
            else:
                if redirect_count == self.MAX_REDIRECTS:
                    self.set_error('redirect-limit-reached')
                    return None
                redirect_count += 1

                location = _response.getheaders().get('location', None)
                if location is None or location.strip() == '':
                    self.set_error('redirect-empty-location')
                    return None
                ourl = urlparse.urlparse(_url)
                lurl = urlparse.urlparse(location)

                if (lurl.scheme == '' or lurl.scheme == ourl.scheme) \
                        and (lurl.netloc == '' or lurl.netloc == ourl.netloc) \
                        and lurl.path == ourl.path:
                    self.set_error('redirect-loop')
                    return None

                if not self._explicitly_allowed_redirect(ourl, lurl):
                    if lurl.netloc and lurl.netloc.lower() != ourl.netloc.lower():
                        msg = 'redirect-host-mismatch {0}'.format(location)
                        self.set_error(msg)
                        return None

                    if ourl.scheme.lower() == 'http' and lurl.scheme.lower() == 'https':
                        #
                        # Allow http to https but nothing else.
                        #
                        pass
                    elif lurl.scheme and lurl.scheme.lower() != ourl.scheme.lower():
                        msg = 'redirect-host-mismatch {0}'.format(location)
                        self.set_error(msg)
                        return None

                #
                # Create new url by updating path, query and fragment.
                #
                scheme, netloc, path, query, fragment = \
                    urlparse.urlsplit(_url)
                parts = (
                    lurl.scheme or scheme,
                    lurl.netloc or netloc,
                    lurl.path,
                    lurl.query,
                    lurl.fragment
                )
                _url = urlparse.urlunsplit(parts)

        if read_response:
            _body = _response.data

        content_type = _response.headers.get('content-type', None)
        charset = None
        if content_type:
            w = content_type.split(';')
            content_type = w[0]
            for s in w[1:]:
                s = s.strip()
                if s.startswith('charset='):
                    charset = s.split('charset=', 1)[1]
        if charset:
            _body_unicode = codecs.decode(_body, charset, 'ignore')
        else:
            _body_unicode = codecs.decode(_body, 'Latin-1', 'ignore')
            #_body_unicode = unicode(_body)

        if read_response and hasattr(self.options, 'tag_lines') and self.options.tag_lines:
            tag = self._url + ':'
            _body = '\n' + tag + _body.replace('\n', '\n' + tag)

        result['host'] = _host
        result['url'] = _url
        result['response'] = _response
        result['code'] = _code
        result['msg'] = _msg
        result['body'] = _body
        result['body_unicode'] = _body_unicode
        return result

    def __format__(self, line, parse_result, action_result):
        options = self.options
        url = action_result['url']
        response = action_result['response']
        code = action_result['code']
        msg = action_result['msg']
        output = ''
        #
        if options.server:
            server = response.headers.get('server', None)
            output = '{0} {1}'.format(url, server)
        elif options.code:
            if options.verbose:
                output = '{0} {1} {2}'.format(url, code, msg)
            else:
                output = '{0} {1}'.format(url, code)
        elif options.xpath:
            try:
                import lxml.etree
            except ImportError:
                import time
                errmsg = '\n'
                errmsg += '*** XNET USAGE INFORMATION ***\n'
                errmsg += '\n'
                errmsg += 'ImportError raised, lxml.etree might be missing.\n'
                errmsg += 'For Debian, try:\n'
                errmsg += '  $ sudo apt-get install python-lxml\n'
                errmsg += 'For OSX, try:\n'
                errmsg += '  $ sudo port install py-lxml\n'
                print errmsg
                time.sleep(3)
                raise
            htmlparser = lxml.etree.HTMLParser()
            tree = lxml.etree.fromstring(action_result['body_unicode'], htmlparser)
            for xpath in self.options.xpath:
                xpath_result = tree.xpath(xpath)
                success = False
                for elem in xpath_result:
                    success = True
                    if type(elem) is lxml.etree._ElementStringResult:
                        elem = str(elem)
                    if not type(elem) is str:
                        elem = lxml.etree.tostring(elem)
                    output += elem
                    if len(output) and output[-1] != '\n':
                        output += '\n'
                if self.options.xpath_strict and not success:
                    import tempfile
                    (fd, name) = tempfile.mkstemp(prefix='webget-xpath-fail-')
                    os.write(fd, action_result['body'])
                    os.close(fd)
                    msg = 'Error: XPath not matched: {0}'.format(xpath)
                    msg += ', body saved to {0}'.format(name)
                    sys.exit(msg)
                output += self.options.xpath_delimiter or ' '
        else:
            output = action_result['body']
        return output

    @classmethod
    def __format_help__(cls):
        output = '''
Format variables for %s:

 host     - target host
 url      - target URL
 code     - status code
 msg      - status message
 body     - response body
 time     - time consumed by action, in ms

Default format depend on commandline arguments.
        ''' % cls.__toolname__
        return output

    def __timeout__(self):
        return 'timeout {0}\n'.format(self._url)


def main():
    import xnet.tools.run
    xnet.tools.run.run(WebgetTool)


if __name__ == "__main__":
    main()
    #doctest.testmod()
    #unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright(c) 2013 Krister Hedfors
#

__all__ = [
    'Tool',  # refactor >> InputHandler
]

import unittest
import doctest
import re
import time

import gevent

from xnet.net.ipv4 import AddressIterator


class ToolException(Exception):
    pass


class NotImplementedException(ToolException):
    pass


class InvalidParseResult(ToolException):
    pass


class InvalidToolResult(ToolException):
    pass


class NoResultAvailable(ToolException):
    pass


class BadFormat(ToolException):
    pass


class Formatter(object):
    '''
        Formatter
            Behaves as string.format but allows loops.
            >>> fmt = Formatter('for i in col: yay {i}, ')
            >>> fmt.format(col=[1, 2, 3])
            'yay 1, yay 2, yay 3, '
    '''

    _loop_stmt = re.compile(r'^for (?P<varname>\w+) in (?P<colname>\w+):(?P<text>.*)')

    def __init__(self, fstr):
        self._fstr = fstr

    def format(self, *args, **kw):
        return self._render(self._fstr, *args, **kw)

    def _render(self, fstr, *args, **kw):
        m = self._loop_stmt.match(fstr)
        if m:
            return self._render_loop(m.group('varname'), m.group('colname'),
                                     m.group('text'), *args, **kw)
        return self._format(fstr, *args, **kw)

    def _render_loop(self, varname, colname, text, *args, **kw):
        col = kw[colname]
        output = ''
        for i in col:
            kw[varname] = i
            #
            # make this statement:
            #  'for i in x: blabla..'
            # behave as this:
            #  'for i in x:blabla..'
            #
            if len(text) > 1 and text[0] == ' ':
                text = text[1:]

            output += self._render(text, *args, **kw)
        return output

    def _format(self, fstr, *args, **kw):
        fstr = fstr.replace('\\n', '\n')
        fstr = fstr.replace('\\t', '\t')
        return fstr.format(*args, **kw)


class Test_Formatter(unittest.TestCase):

    def test_basic(self):
        f = Formatter('asd {0} foo {BAR}')
        r = f.format(123, BAR='bar')
        self.assertEqual(r, 'asd 123 foo bar')

    def test_loop_1(self):
        f = Formatter('for n in numbers: {n},')
        r = f.format(numbers=[1, 2, 3])
        self.assertEqual(r, '1,2,3,')

    def test_loop_2(self):
        f = Formatter('for c in chars: for n in numbers: {c}{n},')
        r = f.format(chars=['a', 'b'], numbers=[1, 2, 3])
        self.assertEqual(r, 'a1,a2,a3,b1,b2,b3,')


class ToolError:
    codes = {
        0: 'CustomError',
        1: 'ParseFailed',
        2: 'ActionFailed',
        3: 'WaitTimeout',
    }
    CustomError = 0
    ParseFailed = 1
    ActionFailed = 2

    def __init__(self, code, message=None):
        self.code = code
        self.message = message or self.strerror

    @property
    def strerror(self):
        return self.codes[self.code]

    def __str__(self):
        return self.message


class WaitTimeout:
    pass


class Tool(object):
    '''
        Must implement __action__, all other methods are optional.
        By default, __action__ is called with a single-value dict:
            {'item': item}

        Implementations of __parse__, __action__, __filter__ or __format__
        may use set_error(...) to define arbitrary errors, later readable
        through the 'error' property, which if evaluates as True means
        that action value is not available.

        @classmethod
        __setup__(options)
            input: commandline options object
            output: None

                Prepare class and verify command-line options, before
                input parsing and spawning instances of class.

        @classmethod
        __massage__(iterator, options)
            input: iterator with all input
            output: generate (yield) items to __parse__()

                Used to split or skip certain input items.

        __parse__(item)
            input: item or line of input
            output: dict or None

                None prevents all following method calls and doesn't produce output.

        __action__(parse_result)
            input: dict
            output: dict or None

                None prevents all following method calls and doesn't produce output.
                Each value in output dict should have a corresponding 'vN' value
                where N is the precedence order of the value. This makes
                user-controlled output formatting easier.
                Example:
                    d['v0'] = d['host'] = '1.2.3.4'
                    d['v1'] = d['port'] = '80'

        __filter__(action_result)
            input: dict
            output: bool

                Used to limit output depending on cmdline options, for
                instance.

        __format__(line, parse_result, action_result)
            input: str, dict, dict
            output: str or None

                None prevents all following actions and doesn't produce output.
                Terminating newline may be omitted to produce output from multiple
                invoked action on a single line.

        @classmethod
        __format_help__()
            output: format help string

                Implement to provide output formatting specific help for this
                tool.

        __timeout__()
            output: str or None

                Used to produce output if user-controlled timeout occurrs (-w)
                when __format__() cannot be used due to lack of required input.
                Default __timeout__() returns None.
                May set self.stderr to True to force output to stderr.

        __filename__()
            output: str

                Used to produce file paths for --split-output and --tee-output


    '''
    PRE_CALL = 0
    POST_CALL = 1

    MODE_SINGLE = 0x00
    MODE_GEVENT = 0x01

    def __init__(self, options, mode=MODE_GEVENT):
        self._options = options
        self._mode = mode
        self._parse_result = None
        self._action_result = None
        self._state = self.PRE_CALL
        self._error = None
        self._action_starttime = None
        self._action_endtime = None
        self._cleanup_funcs = []
        self.stderr = False  # if True, print output to stderr
        self._validate_implementations()

    @classmethod
    def _print_source(cls, fname):
        if fname.endswith('.pyc'):
            fname = fname[:-1]
        assert fname.endswith('.py')
        print open(fname).read()

    def _validate_implementations(self):
        #
        # assert __massage__ is classmethod generator
        #
        pass

    def __call__(self, item, iterator):
        '''
            always returns self
        '''
        result = self
        timeout = None
        if not self.options.wait is None:
            wait = float(self.options.wait)
            timeout = gevent.Timeout(wait)
        try:
            try:
                result = self._call(item, iterator)
            except gevent.Timeout, t:
                if t is not timeout:
                    raise
                #
                # Do some handling, if Timeout works...
                #
            except WaitTimeout:
                self.set_error(WaitTimeout)
            finally:
                if timeout:
                    timeout.cancel()
        finally:
            self._do_cleanup()
        return result

    def _call(self, item, iterator):
        '''
            FIXME: how handle empty result from __parse__?

            always returns self
        '''
        self._item = str(item).strip()  # used in filename by --split-output
        parse_result = None
        action_result = None
        #
        # sanity
        #
        assert self._state == self.PRE_CALL
        assert self._error is None
        #
        # __parse__
        #
        parse_result = self.__parse__(item, iterator)
        if parse_result is None or not self.error is None:
            if self.error is None:
                code = ToolError.ParseFailed
                error = ToolError(code)
                self.set_error(error)
            return None
        if not isinstance(parse_result, dict):
            msg = '__parse__() did not return a dict subclass.'
            raise InvalidParseResult(msg)
        #
        # __action__
        #
        self._action_starttime = time.time()
        action_result = self.__action__(parse_result)
        self._action_endtime = time.time()
        if action_result is None or not self.error is None:
            if self.error is None:
                print 'kk error IS NONE', parse_result
                code = ToolError.ActionFailed
                error = ToolError(code)
                self.set_error(error)
            return None
        if not isinstance(action_result, dict):
            msg = '__action__() did not return a dict subclass.'
            raise InvalidToolResult(msg)

        #
        # Some action_results should always be available:
        #
        t = 1000 * (self._action_endtime - self._action_starttime)
        action_result['time'] = '%.1f' % t

        #
        # __filter__
        #
        filter_result = self.__filter__(action_result)
        if not filter_result:
            if not self.error is None:
                msg = '__filter__() may not set error.'
                msg += '. Error set: ' + str(self.error)
                raise ToolException(msg)
            action_result = None
        self._parse_result = parse_result
        self._action_result = action_result
        self._state = self.POST_CALL
        return self

    def _register_cleanup(self, func):
        self._cleanup_funcs.append(func)

    def _do_cleanup(self):
        for func in self._cleanup_funcs:
            func()

    @property
    def value(self):
        '''
            value()
                value may be None only if __action__() succeeds but
                __filter__() returns False.
        '''
        if not self.error is None:
            errmsg = 'value not available, check error'
            raise ToolException(errmsg)
        value = self._action_result
        return value

    @property
    def action_time(self):
        if None in (self._action_starttime, self._action_endtime):
            return None
        return self._action_endtime - self._action_starttime

    @property
    def error(self):
        return self._error

    def set_error(self, error, oneword=False):
        if oneword:
            error = str(error).lower()
            error = error.replace(' ', '-')
        self._error = error

    def __str__(self):
        '''
            __str__()
                Relies on user-implemented __format__().
        '''
        if not self.finished:
            errmsg = 'action not done yet'
            raise NoResultAvailable(errmsg)
        if self._action_result is None:
            errmsg = 'result has been rejected by __filter__()'
            raise NoResultAvailable(errmsg)
        if self.options.format:
            output = self._user_format(
                self.options.format,
                self._action_result
            )
        else:
            output = self.__format__(
                self._item,
                self._parse_result,
                self._action_result)
        return output

    def _user_format(self, fmt, fmt_kwargs):
        '''
            fmt is commandline argument -f
        '''
        output = ''
        try:
            output = Formatter(fmt).format(**fmt_kwargs)
        except KeyError, e:
            bad_key = e.args[0]
            valid_keys = ', '.join(fmt_kwargs.keys())
            errmsg = "invalid format key '{0}', valid keys are: {1}".format(
                bad_key,
                valid_keys
            )
            raise BadFormat(errmsg)
        return output

    def __getitem__(self, name):
        if not self._action_result:
            msg = 'Tool has not yet produced a result.'
            raise NoResultAvailable(msg)
        return self._action_result[name]

    @property
    def options(self):
        return self._options

    @property
    def finished(self):
        assert self._state in (self.PRE_CALL, self.POST_CALL)
        return self._state == self.POST_CALL

    #
    # __setup__ optional preparations before we start parsing input and spawning
    # instances.
    #
    @classmethod
    def __setup__(cls, options):
        pass

    #
    # __massage__ must return iterator of items
    #
    @classmethod
    def __massage__(self, iterator, options):
        return iterator

    #
    # __parse__ must return either a dict or None.
    #
    def __parse__(self, item, iterator):
        raise NotImplementedException('Please implement Tool.__parse__.')

    #
    # __action__ is called with whatever __parse__ returned.
    #
    def __action__(self, parse_result):
        raise NotImplementedException('Please implement Tool.__action__.')

    #
    # __format__ must be implemented in order to make __str__() work.
    #
    def __format__(self, item, parse_result, action_result):
        raise NotImplementedException('Please implement Tool.__format__.')

    #
    # __filter__ may return None to filter items
    #
    def __filter__(self, action_result):
        return action_result

    #
    # __timeout__ is used instead of __format__ when user-controlled
    #             timeout occurrs.
    #
    def __timeout__(self):
        return None

    #
    # __format_help__ prints output format specific help
    #
    @classmethod
    def __format_help__(cls):
        return 'Not implemented'

    #
    # Should generally be implemented by Tools
    # to avoid unexpected behaviour.
    #
    def __filename__(self):
        name = str(self._item).strip()
        name = name.replace('/', '__')
        name = name.replace('\\', '--')
        return name


class ToolMixin_AddrIterator(object):
    '''
        This iterator handles hosts and IP-ranges,
        with optional port ranges, such as:
            10.0.0.0/24
            10.0.0.0/255.255.255.0
            10.0.*.5-8
            192.168.0.5:445
            192.168.0.5:22,23,22-100
            192.168.0.0/28:22,23,22-100
            172.20.1-3.*:1-1024,8080

        And argument-specified ports such as:
            -p 3306
            -p 22,23,25
            -p 22-100
            -p 80,443,8080-8089

        Note: hosts with explicitly specified ports (such as
        192.168.0.1:445) will never be yielded with any other
        ports than 445. Colon-specified port ranges override
        argument-specified ranges.
    '''
    @classmethod
    def __massage__(cls, iterator, options):
        ai = AddressIterator(options.port or '80')
        for xaddr in iterator:
            for addr in ai.expand(xaddr):
                yield addr

    def __parse__(self, item, iterator):
        host_port = item.strip()
        (host, port) = host_port.split(':')
        addr = (host, int(port))
        self._addr = addr  # for __timeout__()
        result = {'addr': addr}
        return result

class Test1(unittest.TestCase):
    def test_file(self):
        pass


if __name__ == "__main__":
    doctest.testmod()
    unittest.main()
